#!/usr/bin/env python
# -*- coding: utf-8 -*-

# PyPreTeX - a Pythonic Previewer for LaTeX
# (c) 2007 Alexandre Erwin Ittner <alexandre@ittner.com.br>
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston,
# MA 02111-1307, USA.

"""
Main application module.
"""


import pygtk
import gtk
from gtk import gdk

import os
import tempfile
import subprocess
import shutil

import utils
import pngcomments



class BaseDialog(object):
    UI_FILE_NAME = 'pypretex.ui'

    def ui_init(self, root=None):
        fname = utils.search_share_path(BaseDialog.UI_FILE_NAME)
        self.builder = gtk.Builder()
        self.builder.add_from_file(fname)
        if root:
            return self.builder.get_object(root)


class AboutDialog(BaseDialog):

    def __init__(self, parent):
        self.parent = parent
        self.dlg = self.ui_init("dlgAbout")
        self.builder.connect_signals(self)

    def show(self):
        self.dlg.set_transient_for(self.parent)
        self.dlg.set_modal(True)
        self.dlg.run()

    def on_cmd_close(self, widget = None, data = None):
        self.dlg.destroy()



class PyPreTeX(BaseDialog):

    LATEX_COMMAND = '/usr/bin/latex'
    DVIPNG_COMMAND = '/usr/bin/dvipng'
    TEX_FILE_NAME = 'out'

    LATEX_START = u"""
\\documentclass[14pt]{letter}
\\usepackage[utf8]{inputenc}
\\pagestyle{empty}
\\setlength{\\oddsidemargin}{0cm}
\\setlength{\\evensidemargin}{0cm}
\\setlength{\\topmargin}{-2.0cm}
\\begin{document}
"""
    LATEX_END = u"""
\\end{document}
"""
    
    def __init__(self):
        self.has_img = False
        self.tmp_dir = tempfile.mkdtemp(u".tmp", u"pypretex.")
        self.ui_init()
        self.builder.connect_signals(self)
        self.window = self.builder.get_object("wndMain")
        self.txInput = self.builder.get_object("txInput")
        self.vpOutput = self.builder.get_object("vpOutput")
        self.imgOutput = self.builder.get_object("imgOutput")
        self.txOutput = self.builder.get_object("txOutput")
        self.expOutput = self.builder.get_object("expOutput")
        self.inputBuffer = self.txInput.get_buffer()
        self.outputBuffer = self.txOutput.get_buffer()
        self.vpOutput.modify_bg(gtk.STATE_NORMAL, gdk.color_parse("white"))

        self.endMark = None
        self.txOutput.set_cursor_visible(False)
        self.pngname = None
        self.inputBuffer.set_text("$$")


    def start(self):
        self.window.show()
        self.log("TMP_DIR = " + self.tmp_dir + "\n")
        self.log("PyPreTeX ready.\n")

    def on_cmd_about(self, widget = None, data = None):
        dlg = AboutDialog(self.window)
        dlg.show()

    def on_cmd_quit(self, widget = None, data = None):
        shutil.rmtree(self.tmp_dir, True)
        self.window.destroy()
        gtk.main_quit()

    def on_cmd_import(self, widget = None, data = None):
        dlg = gtk.FileChooserDialog(u"Import TeX code from image", None,
            gtk.FILE_CHOOSER_ACTION_OPEN,
            (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
            gtk.STOCK_OK, gtk.RESPONSE_OK))

        dlg.set_default_response(gtk.RESPONSE_OK)

        filter = gtk.FileFilter()
        filter.set_name("PNG Image")
        filter.add_pattern("*.png")
        dlg.add_filter(filter)

        while True:
            ret = dlg.run()
            if ret == gtk.RESPONSE_OK:
                fname = dlg.get_filename()
                chunks = pngcomments.read_text_chunks(fname)
                for ck in chunks:
                    if len(ck) and ck[0] == 'PyPreTeXData':
                        dlg.destroy()
                        text = ck[1].decode("utf-8")
                        self.inputBuffer.set_text(text)
                        self.imgOutput.set_from_file(fname)
                        return
                utils.error_dialog(dlg, "There is no LaTeX code in this "
                    + "file. It usually means that the file was not created "
                    + "by PyPreTeX or it was edited with a program that "
                    + "removed the code.")
            else:
                dlg.destroy()
                return

    def on_cmd_save(self, widget = None, data = None):
        self.on_cmd_make()
        if not self.pngname:
            return

        dlg = gtk.FileChooserDialog(u"Save image", None,
            gtk.FILE_CHOOSER_ACTION_SAVE,
            (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
            gtk.STOCK_OK, gtk.RESPONSE_OK))

        dlg.set_default_response(gtk.RESPONSE_OK)
        
        filter = gtk.FileFilter()
        filter.set_name("PNG Image")
        filter.add_pattern("*.png")
        dlg.add_filter(filter)
       
        while True:
            ret = dlg.run()
            if ret == gtk.RESPONSE_OK:
                fname = dlg.get_filename()
                prename, ext = os.path.splitext(fname)
                if ext == "":
                    fname = fname + ".png"
                if os.path.exists(fname):
                    if not utils.confirm_file_overwrite(dlg, fname):
                        continue
                dlg.destroy()
                break
            else:
                dlg.destroy()
                return
        try:
            # Copies the image and adds the metadata to future re-imports.
            pngcomments.write_text_chunks(self.pngname, fname,
                [ [ 'Software', 'PyPreTeX' ],
                [ 'PyPreTeXData', self.texcode.encode("utf-8") ] ], True)
        except:
            print(self.pngname, fname)
            utils.error_dialog(self.window, u"Failed to save the file.")
        

    def on_cmd_make(self, widget = None, data = None):
        self.outputBuffer.set_text("")
        self.texcode = self.get_input_text()
        tex = PyPreTeX.LATEX_START + self.texcode + PyPreTeX.LATEX_END
        fname = os.path.join(self.tmp_dir, PyPreTeX.TEX_FILE_NAME)
        texname = fname + ".tex"
        dviname = fname + ".dvi"
        pngname = fname + "1.png"

        try:
            fp = open(texname, "w")
            fp.write(tex.encode("utf-8"))
            fp.close()
        except:
            self.log_error(u"Can't write output to " + texname)
            return
        try:
            args = [ PyPreTeX.LATEX_COMMAND, "-halt-on-error", texname ]
            self.log(u"Running '" + (u" ").join(args) + u"'")
            proc = subprocess.Popen(args, stdout = subprocess.PIPE,
                    stderr = subprocess.STDOUT, cwd = self.tmp_dir)
            if proc.pid:
                self.log(" (pid: " + str(proc.pid) + ")")
            self.log("\n")
            res = None
            while True:
                line = proc.stdout.readline()
                self.log(line)
                res = proc.poll()
                if res != None:
                    break
            if res != 0:
                self.log_error(u"LaTeX error.\n")
                return
            self.log(u"LaTeX done.\n")
            if not os.path.exists(dviname):
                self.log_error(u"Can't found file " + dviname + u"\n")
                return

            args = [ PyPreTeX.DVIPNG_COMMAND, dviname ]
            self.log(u"Running '" + (u" ").join(args) + u"'")
            proc = subprocess.Popen(args, stdout = subprocess.PIPE,
                    stderr = subprocess.STDOUT, cwd = self.tmp_dir)
            if proc.pid:
                self.log(" (pid: " + str(proc.pid) + ")")
            self.log("\n")
            res = None
            while True:
                line = proc.stdout.readline()
                self.log(line)
                res = proc.poll()
                if res != None:
                    break
            if res != 0:
                self.log_error(u"dvipng error.\n")
                return
            if not os.path.exists(pngname):
                self.log_error(u"Can't found file " + pngname + u"\n")
                return

        except:
            self.log_error(u"Unknown error.\n")
            return

        self.imgOutput.set_from_file(pngname)
        self.pngname = pngname
        self.log(u"Done.\n")

    def get_input_text(self):
        start, end = self.inputBuffer.get_bounds()
        return self.inputBuffer.get_text(start, end).decode("utf-8")

    def log(self, s):
        if not s: return
        self.outputBuffer.insert(self.outputBuffer.get_end_iter(), s)
        if self.endMark:
            self.outputBuffer.move_mark(self.endMark,
                self.outputBuffer.get_end_iter())
        else:
            self.endMark = self.outputBuffer.create_mark("end",
                self.outputBuffer.get_end_iter())
        self.txOutput.scroll_to_mark(self.endMark, 0.1)

    def log_error(self, s):
        self.log(u"\n*ERROR: " + s)
        self.expOutput.set_expanded(True)



if __name__ == '__main__':
    app = PyPreTeX()
    app.start()
    gtk.main()


